@@ -1,19 +1,9 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 1999 Matthias Ettrich (ettrich@kde.org)
-    Copyright (C) 2007 Lubos Lunak (l.lunak@kde.org)
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1999 Matthias Ettrich <ettrich@kde.org>
+    SPDX-FileCopyrightText: 2007 Lubos Lunak <l.lunak@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 /*
  * kwindowsystem.h. Part of the KDE project.
@@ -22,11 +12,11 @@
 #ifndef KWINDOWSYSTEM_H
 #define KWINDOWSYSTEM_H
 
-#include <kwindowsystem_export.h>
 #include <QObject>
 #include <QWidgetList> //For WId
-#include <netwm_def.h>
 #include <kwindowinfo.h>
+#include <kwindowsystem_export.h>
+#include <netwm_def.h>
 
 class KWindowSystemPrivate;
 class NETWinInfo;
@@ -41,7 +31,7 @@ class NETWinInfo;
  * using a more high-level interface than the NETWinInfo/NETRootInfo
  * lowlevel classes.
  *
- * Because of limitiations of the way Qt is implemented on Mac OSX, the WId's
+ * Because of limitations of the way Qt is implemented on Mac OSX, the WId's
  * returned by methods in this class are not compatible with those expected
  * by other Qt methods. So while it should be fine to pass WId's retrieved by
  * for example calling the winId method on a QWidget to methods in this class
@@ -54,6 +44,8 @@ class NETWinInfo;
 class KWINDOWSYSTEM_EXPORT KWindowSystem : public QObject, public NET
 {
     Q_OBJECT
+    Q_PROPERTY(bool isPlatformWayland READ isPlatformWayland CONSTANT)
+    Q_PROPERTY(bool isPlatformX11 READ isPlatformX11 CONSTANT)
 
 public:
     /**
@@ -62,22 +54,22 @@ public:
     static KWindowSystem *self();
 
     /**
-      * Returns the list of all toplevel windows currently managed by the
-      * window manager in the order of creation. Please do not rely on
-      * indexes of this list: Whenever you enter Qt's event loop in your
-      * application, it may happen that entries are removed or added.
-      * Your module should perhaps work on a copy of this list and verify a
-      * window with hasWId() before any operations.
-      *
-      * Iteration over this list can be done easily with
-      * \code
-      *  QList<WId> windows = KWindowSystem::windows();
-      *  for (auto it = windows.cbegin(), end = windows.cend(); it != end; ++it) {
-      *     ... do something here,  (*it) is the current WId.
-      *  }
-      * \endcode
-      * @return the list of all toplevel windows
-      */
+     * Returns the list of all toplevel windows currently managed by the
+     * window manager in the order of creation. Please do not rely on
+     * indexes of this list: Whenever you enter Qt's event loop in your
+     * application, it may happen that entries are removed or added.
+     * Your module should perhaps work on a copy of this list and verify a
+     * window with hasWId() before any operations.
+     *
+     * Iteration over this list can be done easily with
+     * \code
+     *  QList<WId> windows = KWindowSystem::windows();
+     *  for (auto it = windows.cbegin(), end = windows.cend(); it != end; ++it) {
+     *     ... do something here,  (*it) is the current WId.
+     *  }
+     * \endcode
+     * @return the list of all toplevel windows
+     */
     static QList<WId> windows();
 
     /**
@@ -87,7 +79,7 @@ public:
      **/
     static bool hasWId(WId id);
 
-#ifndef KWINDOWSYSTEM_NO_DEPRECATED
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Returns information about window @p win. It is recommended to check
      * whether the returned info is valid by calling the valid() method.
@@ -97,9 +89,10 @@ public:
      *    in the returned data, but make this function faster when not all data is needed.
      * @param properties2 additional properties (see NET::Property2 enum)
      * @return the window information
-     * @deprecated use KWindowInfo directly
+     * @deprecated Since 5.0, use KWindowInfo directly
      */
-    KWINDOWSYSTEM_DEPRECATED static KWindowInfo windowInfo(WId win, NET::Properties properties, NET::Properties2 properties2 = NET::Properties2());
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowInfo(WId, NET::Properties, NET::Properties2")
+    static KWindowInfo windowInfo(WId win, NET::Properties properties, NET::Properties2 properties2 = NET::Properties2());
 #endif
 
     /**
@@ -145,6 +138,36 @@ public:
      */
     static void activateWindow(WId win, long time = 0);
 
+    /**
+     * Requests that window @p win is activated.
+     *
+     * There are two ways how to activate a window, by calling
+     * activateWindow() and forceActiveWindow(). Generally,
+     * applications shouldn't make attempts to explicitly activate
+     * their windows, and instead let the user to activate them.
+     * In the special cases where this may be needed, applications
+     * should use activateWindow(). Window manager may consider whether
+     * this request wouldn't result in focus stealing, which
+     * would be obtrusive, and may refuse the request.
+     *
+     * The usage of forceActiveWindow() is meant only for pagers
+     * and similar tools, which represent direct user actions
+     * related to window manipulation.
+     * Except for rare cases, this request will be always honored,
+     * and normal applications are forbidden to use it.
+     *
+     * In case of problems, consult the KWin README in the kdebase
+     * package (kdebase/kwin/README), or ask on the kwin@kde.org
+     * mailing list.
+     *
+     * @param window the window to make active
+     * @param time X server timestamp of the user activity that
+     *    caused this request
+     *
+     * @since 5.89
+     */
+    Q_INVOKABLE static void activateWindow(QWindow *window, long time = 0);
+
     /**
      * Sets window @p win to be the active window. Note that this
      * should be called only in special cases, applications
@@ -156,7 +179,7 @@ public:
      * @param win the id of the window to make active
      * @param time X server timestamp of the user activity that
      *    caused this request
-    */
+     */
     static void forceActiveWindow(WId win, long time = 0);
 
     /**
@@ -224,6 +247,20 @@ public:
      */
     static void setOnActivities(WId win, const QStringList &activities);
 
+    /**
+     * Sets the parent window of @p subwindow to be @p mainwindow.
+     * This overrides the parent set the usual way as the QWidget or QWindow parent,
+     * but only for the window manager - e.g. stacking order and window grouping
+     * will be affected, but features like automatic deletion of children
+     * when the parent is deleted are unaffected and normally use
+     * the QObject parent.
+     *
+     * This function should be used before a dialog is shown for a window
+     * that belongs to another application.
+     */
+    static void setMainWindow(QWindow *subwindow, WId mainwindow);
+
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 62)
     /**
      * Sets the parent window of @p subwindow to be @p mainwindow.
      * This overrides the parent set the usual way as the QWidget parent,
@@ -234,24 +271,32 @@ public:
      *
      * This function should be used before a dialog is shown for a window
      * that belongs to another application.
+     * @deprecated since 5.62, use setMainWindow(QWindow *). If all you have is a QWidget*,
+     * you might need to call setAttribute(Qt::WA_NativeWindow, true); before calling
+     * >window()->windowHandle().
      */
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 62, "Use KWindowSystem::setMainWindow(QWindow *)")
     static void setMainWindow(QWidget *subwindow, WId mainwindow);
-#ifndef KWINDOWSYSTEM_NO_DEPRECATED
+#endif
+
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Returns the WM_TRANSIENT_FOR property for the given window, i.e. the mainwindow
      * for this window.
      *
      * @param window the id of the window
-     * @deprecated Use KWindowInfo::transientFor
+     * @deprecated Since 5.0, use KWindowInfo::transientFor
      */
-    KWINDOWSYSTEM_DEPRECATED static WId transientFor(WId window);
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowInfo::transientFor()")
+    static WId transientFor(WId window);
 
     /**
      * Returns the leader window for the group the given window is in, if any.
      * @param window the id of the window
-     * @deprecated Use KWindowInfo::groupLeader
+     * @deprecated Since 5.0, use KWindowInfo::groupLeader
      */
-    KWINDOWSYSTEM_DEPRECATED static WId groupLeader(WId window);
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowInfo::groupLeader()")
+    static WId groupLeader(WId window);
 #endif
     /**
      * Returns an icon for window @p win.
@@ -279,11 +324,12 @@ public:
      * @li ClassHint load icon after getting name from the classhint
      * @li XApp load the standard X icon (last fallback)
      */
-    enum IconSource { NETWM = 1, //!< read from property from the window manager specification
-                      WMHints = 2, //!< read from WMHints property
-                      ClassHint = 4, //!< load icon after getting name from the classhint
-                      XApp = 8 //!<load the standard X icon (last fallback)
-                    };
+    enum IconSource {
+        NETWM = 1, //!< read from property from the window manager specification
+        WMHints = 2, //!< read from WMHints property
+        ClassHint = 4, //!< load icon after getting name from the classhint
+        XApp = 8, //!< load the standard X icon (last fallback)
+    };
     /**
      * @overload
      *
@@ -331,7 +377,7 @@ public:
      * @param icon the new icon
      * @param miniIcon the new mini icon
      */
-    static void  setIcons(WId win, const QPixmap &icon, const QPixmap &miniIcon);
+    static void setIcons(WId win, const QPixmap &icon, const QPixmap &miniIcon);
     /**
      * Sets the type of window @p win to @p windowType.
      *
@@ -345,7 +391,7 @@ public:
      * Possible values are or'ed combinations of NET::Modal,
      * NET::Sticky, NET::MaxVert, NET::MaxHoriz, NET::Shaded,
      * NET::SkipTaskbar, NET::SkipPager, NET::Hidden,
-     * NET::FullScreen, NET::KeepAbove, NET::KeepBelow, NET::StaysOnTop,
+     * NET::FullScreen, NET::KeepAbove, NET::KeepBelow,
      * NET::SkipSwitcher
      *
      * @param win the id of the window
@@ -359,13 +405,13 @@ public:
      * Possible values are or'ed combinations of NET::Modal,
      * NET::Sticky, NET::MaxVert, NET::MaxHoriz, NET::Shaded,
      * NET::SkipTaskbar, NET::SkipPager, NET::Hidden,
-     * NET::FullScreen, NET::KeepAbove, NET::KeepBelow, NET::StaysOnTop,
+     * NET::FullScreen, NET::KeepAbove, NET::KeepBelow,
      * NET::SkipSwitcher
      *
      * @param win the id of the window
      * @param state the flags that will be cleared
      */
-    static void clearState(WId win, NET::States  state);
+    static void clearState(WId win, NET::States state);
 
     /**
      * Minimizes the window with id @p win.
@@ -384,17 +430,19 @@ public:
      **/
     static void unminimizeWindow(WId win);
 
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * @deprecated since 5.0 the @p animation is ignored.
      */
-#ifndef KWINDOWSYSTEM_NO_DEPRECATED
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowSystem::minimizeWindow(WId)")
     static void minimizeWindow(WId win, bool animation);
 #endif
 
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * @deprecated since 5.0 the @p animation is ignored.
      */
-#ifndef KWINDOWSYSTEM_NO_DEPRECATED
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowSystem::unminimizeWindow(WId)")
     static void unminimizeWindow(WId win, bool animation);
 #endif
 
@@ -428,7 +476,7 @@ public:
      *        current desktop
      * @return the size and position of the desktop
      **/
-    static QRect workArea(int desktop = - 1);
+    static QRect workArea(int desktop = -1);
 
     /**
      * Returns the workarea for the specified desktop, or the current
@@ -482,7 +530,7 @@ public:
      */
     static void setUserTime(WId win, long time);
     /**
-     * Sets the strut of window @p win to @p to @p left width
+     * Sets the strut of window @p win to @p left_width
      * ranging from @p left_start to @p left_end on the left edge,
      * and simiarly for the other edges. For not reserving a strut, pass 0 as the width.
      * E.g. to reserve 10x10 square in the topleft corner, use e.g.
@@ -502,9 +550,19 @@ public:
      * @param bottom_start starting x coordinate of the strut at the bottom edge
      * @param bottom_end ending x coordinate of the strut at the bottom edge
      */
-    static void setExtendedStrut(WId win, int left_width, int left_start, int left_end,
-                                 int right_width, int right_start, int right_end, int top_width, int top_start, int top_end,
-                                 int bottom_width, int bottom_start, int bottom_end);
+    static void setExtendedStrut(WId win,
+                                 int left_width,
+                                 int left_start,
+                                 int left_end,
+                                 int right_width,
+                                 int right_start,
+                                 int right_end,
+                                 int top_width,
+                                 int top_start,
+                                 int top_end,
+                                 int bottom_width,
+                                 int bottom_start,
+                                 int bottom_end);
 
     /**
      * Convenience function for setExtendedStrut() that automatically makes struts
@@ -581,6 +639,25 @@ public:
      */
     static QPoint constrainViewportRelativePosition(const QPoint &pos);
 
+    /**
+     * Updates the platform-specific startup id, if any.
+     *
+     * This method is to be called when a running application instance
+     * is reused for handling the request to start this application.
+     * A typical use would be in the handler of the KDBusService activation signal.
+     *
+     * For X11, this updates the id for the Startup Notification protocol,
+     * taking the id from QX11Info::nextStartupId(), if not empty.
+     * For Wayland, this updates the token for the XDG Activation protocol,
+     * taking the token from the "XDG_ACTIVATION_TOKEN" environment variable
+     * and then unsetting it, if not empty.
+     *
+     * @param window the main window (needed by X11 platform)
+     *
+     * @since 5.91
+     */
+    static void updateStartupId(QWindow *window);
+
     /**
      * Enum describing the windowing system platform used by the QGuiApplication.
      * @see platform
@@ -598,7 +675,7 @@ public:
         /**
          * The Wayland windowing system platform.
          **/
-        Wayland
+        Wayland,
     };
     /**
      * Returns the Platform used by the QGuiApplication.
@@ -626,6 +703,34 @@ public:
      **/
     static bool isPlatformWayland();
 
+    /**
+     * Requests an xdg_activation_v1 token for a specific window.
+     *
+     * @param win window in behalf this request is made
+     * @param serial of the event that triggered the request
+     * @param app_id identifier of the application that we are launching
+     *
+     * @see currentSerial
+     * @since 5.83
+     */
+    Q_INVOKABLE static void requestXdgActivationToken(QWindow *win, uint32_t serial, const QString &app_id);
+
+    /**
+     * Sets the @p token that will be used when activateWindow is called next
+     *
+     * @since 5.83
+     */
+    Q_INVOKABLE static void setCurrentXdgActivationToken(const QString &token);
+
+    /**
+     * Offers the seat's current serial
+     *
+     * This will mostly be useful on wayland sessions.
+     *
+     * @since 5.83
+     */
+    Q_INVOKABLE static quint32 lastInputSerial(QWindow *window);
+
 Q_SIGNALS:
 
     /**
@@ -691,9 +796,10 @@ Q_SIGNALS:
      * @param properties2 the properties2 that were modified
      *
      * @since 5.0
-     **/
-    void windowChanged(WId id, NET::Properties properties, NET::Properties2 properties2);
+     */
+    void windowChanged(WId id, NET::Properties properties, NET::Properties2 properties2); // clazy:exclude=overloaded-signal
 
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * The window changed.
      *
@@ -706,28 +812,34 @@ Q_SIGNALS:
      *
      * @deprecated since 5.0 use windowChanged(WId, NET::Properties, NET::Properties2)
      */
-#ifndef KWINDOWSYSTEM_NO_DEPRECATED
-    QT_MOC_COMPAT void windowChanged(WId id, const unsigned long *properties);
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowSystem::windowChanged(WId, NET::Properties, NET::Properties2)")
+    QT_MOC_COMPAT void windowChanged(WId id, const unsigned long *properties); // clazy:exclude=overloaded-signal
 #endif
 
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
-     * @deprecated
      * The window changed.
      *
      * The unsigned int parameter contains the NET properties that
      * were modified (see netwm_def.h).
      * @param id the id of the window
      * @param properties the properties that were modified
+     * @deprecated Since 5.0
      */
-#ifndef KWINDOWSYSTEM_NO_DEPRECATED
-    QT_MOC_COMPAT void windowChanged(WId id, unsigned int properties);
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 0, "Use KWindowSystem::windowChanged(WId, NET::Properties, NET::Properties2)")
+    QT_MOC_COMPAT void windowChanged(WId id, unsigned int properties); // clazy:exclude=overloaded-signal
 #endif
 
+#if KWINDOWSYSTEM_ENABLE_DEPRECATED_SINCE(5, 80)
     /**
      * The window changed somehow.
      * @param id the id of the window
-     */
-    void windowChanged(WId id);
+     *
+     * @deprecated since 5.80, use windowChanged(WId, NET::Properties, NET::Properties2);
+     **/
+    KWINDOWSYSTEM_DEPRECATED_VERSION(5, 80, "Use KWindowSystem::windowChanged(WId, NET::Properties, NET::Properties2)")
+    void windowChanged(WId id); // clazy:exclude=overloaded-signal
+#endif
 
     /**
      * The state of showing the desktop has changed.
@@ -748,13 +860,24 @@ Q_SIGNALS:
      */
     void compositingChanged(bool enabled);
 
+    /**
+     * Activation @p token to pass to the client.
+     *
+     * @see requestXdgActivationToken
+     * @see setCurrentXdgActivationToken
+     * @since 5.83
+     */
+    void xdgActivationTokenArrived(int serial, const QString &token);
+
 protected:
     void connectNotify(const QMetaMethod &signal) override;
 
 private:
     friend class KWindowSystemStaticContainer;
 
-    KWindowSystem() {}
+    KWindowSystem()
+    {
+    }
     static KWindowSystemPrivate *d_func();
 };
 
